--- old/usr/src/uts/common/fs/zfs/ddt.c	Fri Apr 20 04:05:11 2012
+++ new/usr/src/uts/common/fs/zfs/ddt.c	Fri Apr 20 04:05:10 2012
@@ -287,6 +287,7 @@
 void
 ddt_phys_fill(ddt_phys_t *ddp, const blkptr_t *bp)
 {
+	ASSERT(ddp != NULL);
 	ASSERT(ddp->ddp_phys_birth == 0);
 
 	for (int d = 0; d < SPA_DVAS_PER_BP; d++)
@@ -297,20 +298,29 @@
 void
 ddt_phys_clear(ddt_phys_t *ddp)
 {
-	bzero(ddp, sizeof (*ddp));
+	ASSERT(ddp != NULL);
+	if (ddp) {
+		bzero(ddp, sizeof (*ddp));
+	}
 }
 
 void
 ddt_phys_addref(ddt_phys_t *ddp)
 {
-	ddp->ddp_refcnt++;
+	ASSERT(ddp != NULL);
+	if (ddp) {
+		ddp->ddp_refcnt++;
+	}
 }
 
 void
 ddt_phys_decref(ddt_phys_t *ddp)
 {
-	ASSERT((int64_t)ddp->ddp_refcnt > 0);
-	ddp->ddp_refcnt--;
+//	ASSERT(ddp != NULL);
+	if (ddp) {
+		ASSERT((int64_t)ddp->ddp_refcnt > 0);
+		ddp->ddp_refcnt--;
+	}
 }
 
 void
@@ -333,6 +343,13 @@
 		    BP_PHYSICAL_BIRTH(bp) == ddp->ddp_phys_birth)
 			return (ddp);
 	}
+	(void) printf("ddt_phys_select() found nothing for "
+	    "DVA[BP]=<%llu:%llx:%llx> and phys_birth[BP]=%llu\n",
+	    (u_longlong_t)DVA_GET_VDEV(BP_IDENTITY(bp)),
+	    (u_longlong_t)DVA_GET_OFFSET(BP_IDENTITY(bp)),
+	    (u_longlong_t)DVA_GET_ASIZE(BP_IDENTITY(bp)),
+	    (u_longlong_t)BP_PHYSICAL_BIRTH(bp)
+	);
 	return (NULL);
 }
 
